import { DocumentModel as InnerDocumentModel } from '@alilc/lowcode-designer';
import { TransformStage, RootSchema } from '@alilc/lowcode-types';
import Node from './node';
import Selection from './selection';
import Detecting from './detecting';
import History from './history';
import Project from './project';
import Prop from './prop';
import Canvas from './canvas';
import ModalNodesManager from './modal-nodes-manager';
import { documentSymbol, editorSymbol } from './symbols';
declare type IOnChangeOptions = {
    type: string;
    node: Node;
};
declare type PropChangeOptions = {
    key?: string | number;
    prop?: Prop;
    node: Node;
    newValue: any;
    oldValue: any;
};
export default class DocumentModel {
    private readonly [documentSymbol];
    private readonly [editorSymbol];
    private _focusNode;
    selection: Selection;
    detecting: Detecting;
    history: History;
    canvas: Canvas;
    constructor(document: InnerDocumentModel);
    static create(document: InnerDocumentModel | undefined | null): DocumentModel;
    /**
     * id
     */
    get id(): string;
    set id(id: string);
    /**
     * 获取当前文档所属的 project
     * @returns
     */
    get project(): Project;
    /**
     * 获取文档的根节点
     * @returns
     */
    get root(): Node | null;
    get focusNode(): Node;
    set focusNode(node: Node);
    /**
     * 获取文档下所有节点
     * @returns
     */
    get nodesMap(): Map<string, Node>;
    /**
     * 模态节点管理
     */
    get modalNodesManager(): ModalNodesManager;
    get dropLocation(): import("../../designer/lib").DropLocation;
    /**
     * 根据 nodeId 返回 Node 实例
     * @param nodeId
     * @returns
     */
    getNodeById(nodeId: string): any;
    /**
     * 导入 schema
     * @param schema
     */
    importSchema(schema: RootSchema): void;
    /**
     * 导出 schema
     * @param stage
     * @returns
     */
    exportSchema(stage?: TransformStage): import("@alilc/lowcode-types").PageSchema | import("@alilc/lowcode-types").ComponentSchema;
    /**
     * 插入节点
     * @param parent
     * @param thing
     * @param at
     * @param copy
     * @returns
     */
    insertNode(parent: Node, thing: Node, at?: number | null | undefined, copy?: boolean | undefined): any;
    /**
     * 创建一个节点
     * @param data
     * @returns
     */
    createNode(data: any): any;
    /**
     * 移除指定节点/节点id
     * @param idOrNode
     */
    removeNode(idOrNode: string | Node): void;
    /**
     * componentsMap of documentModel
     * @param extraComps
     * @returns
     */
    getComponentsMap(extraComps?: string[]): import("../../designer/lib").ComponentMap[];
    /**
     * 当前 document 新增节点事件
     */
    onAddNode(fn: (node: Node) => void): () => void;
    /**
     * 当前 document 新增节点事件，此时节点已经挂载到 document 上
     */
    onMountNode(fn: (node: Node) => void): () => void;
    /**
     * 当前 document 删除节点事件
     */
    onRemoveNode(fn: (node: Node) => void): () => void;
    /**
     * 当前 document 的 hover 变更事件
     */
    onChangeDetecting(fn: (node: Node) => void): () => void;
    /**
     * 当前 document 的选中变更事件
     */
    onChangeSelection(fn: (ids: string[]) => void): () => void;
    /**
     * 当前 document 的节点显隐状态变更事件
     * @param fn
     */
    onChangeNodeVisible(fn: (node: Node, visible: boolean) => void): void;
    /**
     * 当前 document 的节点 children 变更事件
     * @param fn
     */
    onChangeNodeChildren(fn: (info?: IOnChangeOptions) => void): void;
    /**
     * 当前 document 节点属性修改事件
     * @param fn
     */
    onChangeNodeProp(fn: (info: PropChangeOptions) => void): void;
}
export {};
