import _createClass from "@babel/runtime/helpers/createClass";

function _createForOfIteratorHelperLoose(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (it) return (it = it.call(o)).next.bind(it); if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; return function () { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

import { TransformStage, GlobalEvent } from '@alilc/lowcode-types';
import Node from './node';
import Selection from './selection';
import Detecting from './detecting';
import History from './history';
import Project from './project';
import Prop from './prop';
import Canvas from './canvas';
import ModalNodesManager from './modal-nodes-manager';
import { documentSymbol, editorSymbol, nodeSymbol } from './symbols';

var DocumentModel = /*#__PURE__*/function () {
  function DocumentModel(document) {
    this[documentSymbol] = void 0;
    this[editorSymbol] = void 0;
    this._focusNode = void 0;
    this.selection = void 0;
    this.detecting = void 0;
    this.history = void 0;
    this.canvas = void 0;
    this[documentSymbol] = document;
    this[editorSymbol] = document.designer.editor;
    this.selection = new Selection(document);
    this.detecting = new Detecting(document);
    this.history = new History(document.getHistory());
    this.canvas = new Canvas(document.designer);
    this._focusNode = Node.create(this[documentSymbol].focusNode);
  }

  DocumentModel.create = function create(document) {
    if (document == undefined) return null;
    return new DocumentModel(document);
  }
  /**
   * id
   */
  ;

  var _proto = DocumentModel.prototype;

  /**
   * 根据 nodeId 返回 Node 实例
   * @param nodeId
   * @returns
   */
  _proto.getNodeById = function getNodeById(nodeId) {
    return Node.create(this[documentSymbol].getNode(nodeId));
  }
  /**
   * 导入 schema
   * @param schema
   */
  ;

  _proto.importSchema = function importSchema(schema) {
    this[documentSymbol]["import"](schema);
  }
  /**
   * 导出 schema
   * @param stage
   * @returns
   */
  ;

  _proto.exportSchema = function exportSchema(stage) {
    if (stage === void 0) {
      stage = TransformStage.Render;
    }

    return this[documentSymbol]["export"](stage);
  }
  /**
   * 插入节点
   * @param parent
   * @param thing
   * @param at
   * @param copy
   * @returns
   */
  ;

  _proto.insertNode = function insertNode(parent, thing, at, copy) {
    var node = this[documentSymbol].insertNode(parent[nodeSymbol] ? parent[nodeSymbol] : parent, thing !== null && thing !== void 0 && thing[nodeSymbol] ? thing[nodeSymbol] : thing, at, copy);
    return Node.create(node);
  }
  /**
   * 创建一个节点
   * @param data
   * @returns
   */
  ;

  _proto.createNode = function createNode(data) {
    return Node.create(this[documentSymbol].createNode(data));
  }
  /**
   * 移除指定节点/节点id
   * @param idOrNode
   */
  ;

  _proto.removeNode = function removeNode(idOrNode) {
    this[documentSymbol].removeNode(idOrNode);
  }
  /**
   * componentsMap of documentModel
   * @param extraComps
   * @returns
   */
  ;

  _proto.getComponentsMap = function getComponentsMap(extraComps) {
    return this[documentSymbol].getComponentsMap(extraComps);
  }
  /**
   * 当前 document 新增节点事件
   */
  ;

  _proto.onAddNode = function onAddNode(fn) {
    return this[documentSymbol].onNodeCreate(function (node) {
      fn(Node.create(node));
    });
  }
  /**
   * 当前 document 新增节点事件，此时节点已经挂载到 document 上
   */
  ;

  _proto.onMountNode = function onMountNode(fn) {
    var _this = this;

    this[editorSymbol].on('node.add', fn);
    return function () {
      _this[editorSymbol].off('node.add', fn);
    };
  }
  /**
   * 当前 document 删除节点事件
   */
  ;

  _proto.onRemoveNode = function onRemoveNode(fn) {
    return this[documentSymbol].onNodeDestroy(function (node) {
      fn(Node.create(node));
    });
  }
  /**
   * 当前 document 的 hover 变更事件
   */
  ;

  _proto.onChangeDetecting = function onChangeDetecting(fn) {
    return this[documentSymbol].designer.detecting.onDetectingChange(function (node) {
      fn(Node.create(node));
    });
  }
  /**
   * 当前 document 的选中变更事件
   */
  ;

  _proto.onChangeSelection = function onChangeSelection(fn) {
    return this[documentSymbol].selection.onSelectionChange(function (ids) {
      fn(ids);
    });
  }
  /**
   * 当前 document 的节点显隐状态变更事件
   * @param fn
   */
  ;

  _proto.onChangeNodeVisible = function onChangeNodeVisible(fn) {
    // TODO: history 变化时需要重新绑定
    this[documentSymbol].nodesMap.forEach(function (node) {
      node.onVisibleChange(function (flag) {
        fn(Node.create(node), flag);
      });
    });
  }
  /**
   * 当前 document 的节点 children 变更事件
   * @param fn
   */
  ;

  _proto.onChangeNodeChildren = function onChangeNodeChildren(fn) {
    // TODO: history 变化时需要重新绑定
    this[documentSymbol].nodesMap.forEach(function (node) {
      node.onChildrenChange(function (info) {
        return info ? fn({
          type: info.type,
          node: Node.create(node)
        }) : fn();
      });
    });
  }
  /**
   * 当前 document 节点属性修改事件
   * @param fn
   */
  ;

  _proto.onChangeNodeProp = function onChangeNodeProp(fn) {
    this[editorSymbol].on(GlobalEvent.Node.Prop.InnerChange, function (info) {
      fn({
        key: info.key,
        oldValue: info.oldValue,
        newValue: info.newValue,
        prop: Prop.create(info.prop),
        node: Node.create(info.node)
      });
    });
  };

  _createClass(DocumentModel, [{
    key: "id",
    get: function get() {
      return this[documentSymbol].id;
    },
    set: function set(id) {
      this[documentSymbol].id = id;
    }
    /**
     * 获取当前文档所属的 project
     * @returns
     */

  }, {
    key: "project",
    get: function get() {
      return Project.create(this[documentSymbol].project);
    }
    /**
     * 获取文档的根节点
     * @returns
     */

  }, {
    key: "root",
    get: function get() {
      return Node.create(this[documentSymbol].getRoot());
    }
  }, {
    key: "focusNode",
    get: function get() {
      return this._focusNode;
    },
    set: function set(node) {
      this._focusNode = node;
    }
    /**
     * 获取文档下所有节点
     * @returns
     */

  }, {
    key: "nodesMap",
    get: function get() {
      var map = new Map();

      for (var _iterator = _createForOfIteratorHelperLoose(this[documentSymbol].nodesMap.keys()), _step; !(_step = _iterator()).done;) {
        var id = _step.value;
        map.set(id, this.getNodeById(id));
      }

      return map;
    }
    /**
     * 模态节点管理
     */

  }, {
    key: "modalNodesManager",
    get: function get() {
      return ModalNodesManager.create(this[documentSymbol].modalNodesManager);
    } // @TODO: 不能直接暴露

  }, {
    key: "dropLocation",
    get: function get() {
      return this[documentSymbol].dropLocation;
    }
  }]);

  return DocumentModel;
}();

export { DocumentModel as default };