import { Project as InnerProject, PropsReducer as PropsTransducer, TransformStage } from '@alilc/lowcode-designer';
import { RootSchema, ProjectSchema } from '@alilc/lowcode-types';
import DocumentModel from './document-model';
import SimulatorHost from './simulator-host';
import { projectSymbol, simulatorHostSymbol, simulatorRendererSymbol } from './symbols';
export default class Project {
    private readonly [projectSymbol];
    private [simulatorHostSymbol];
    private [simulatorRendererSymbol];
    constructor(project: InnerProject);
    static create(project: InnerProject): Project;
    /**
     * 获取当前的 document
     * @returns
     */
    get currentDocument(): DocumentModel | null;
    /**
     * 获取当前 project 下所有 documents
     * @returns
     */
    get documents(): DocumentModel[];
    /**
     * 获取模拟器的 host
     */
    get simulatorHost(): SimulatorHost;
    /**
     * @deprecated use .simulatorHost instead.
     */
    get simulator(): SimulatorHost;
    /**
     * 打开一个 document
     * @param doc
     * @returns
     */
    openDocument(doc?: string | RootSchema | undefined): DocumentModel;
    /**
     * 创建一个 document
     * @param data
     * @returns
     */
    createDocument(data?: RootSchema): DocumentModel | null;
    /**
     * 删除一个 document
     * @param doc
     */
    removeDocument(doc: DocumentModel): void;
    /**
     * 根据 fileName 获取 document
     * @param fileName
     * @returns
     */
    getDocumentByFileName(fileName: string): DocumentModel | null;
    /**
     * 根据 id 获取 document
     * @param id
     * @returns
     */
    getDocumentById(id: string): DocumentModel | null;
    /**
     * 导出 project
     * @returns
     */
    exportSchema(stage?: TransformStage): ProjectSchema;
    /**
     * 导入 project
     * @param schema 待导入的 project 数据
     */
    importSchema(schema?: ProjectSchema): void;
    /**
     * 获取当前的 document
     * @returns
     */
    getCurrentDocument(): DocumentModel | null;
    /**
     * 增加一个属性的管道处理函数
     * @param transducer
     * @param stage
     */
    addPropsTransducer(transducer: PropsTransducer, stage: TransformStage): void;
    /**
     * 当前 project 内的 document 变更事件
     */
    onChangeDocument(fn: (doc: DocumentModel) => void): () => void;
    /**
     * 当前 project 的模拟器 ready 事件
     */
    onSimulatorHostReady(fn: (host: SimulatorHost) => void): () => void;
    /**
     * 当前 project 的渲染器 ready 事件
     */
    onSimulatorRendererReady(fn: () => void): () => void;
}
